function [x, x_deb]=perform_refitting(x_obs,phi,lambda,block_penalty,nb_iteration)
Nx=size(x_obs,1);
Ny=size(x_obs,2);
%in case one would want to generalize the code and consider a non square matrix phi of size
%(Nx_obs*Ny_obs,Nx*Ny)
Nx_obs=size(x_obs,1);
Ny_obs=size(x_obs,2);
Ncolor=size(x_obs,3);

%% Define Operator gradient and its transpose


K = @(v)[reshape(gradx(v),[Nx*Ny Ncolor]) reshape(grady(v),[Nx*Ny Ncolor])];
KT= @(z)divx(reshape(z(:,1:Ncolor),[Nx Ny Ncolor]))+divy( reshape(z(:,Ncolor+1:2*Ncolor),[Nx Ny Ncolor]));





% Full support with Bregman method
full_support=strcmp(block_penalty,'Bregman');


% if there is an operator in the data term
phi_not_id=0;
if length(phi)>1
    phi_not_id=1;
end

phitphi=phi'*phi;
phi_t_obs=x_obs;
if phi_not_id
    phi_t_obs= reshape(phi'*reshape(x_obs,[Nx_obs*Ny_obs ,Ncolor]), [Nx Ny Ncolor]);
end


mul_phitphi=@(phi,v) reshape(phi*reshape(v,[Nx*Ny ,Ncolor]), [Nx Ny Ncolor]);
Id=speye(Nx*Ny);


%% Prima dual parameters

tau=0.25;
sigma=0.5;
theta=1;

if Ncolor==3
  tau=1./12;  
end

tau_deb=tau;
sigma_deb=sigma;
theta_deb=1;

% initalization of all variables
x=x_obs;
v=x_obs;
x_deb=x_obs;
v_deb=x_obs;
z=K(v)*0;
z_deb=z;


% Get the size of block
nb_blocks=size(z,2);

% Projection of z onto the orientation axis of n
proj_z=@(z,n) repmat(sum(n.*z,2), [1 nb_blocks]).*n;
compute_norm_z=@(z)repmat(sqrt(sum(z.^2,2)), [1 nb_blocks]);
norm_z=compute_norm_z(z);
% Proximal operator for standard dual variable
prox_z= @(z,norm_z,lambda,sigma)z*lambda./max(lambda, norm_z);   %\ell_2 projection on the l_2 ball (=dual of TV) 


%% Go
for t=1:nb_iteration
   
    xold=x;
    xold_deb=x_deb;
    
    
    % Dual updates
    z= z+sigma*K(v);
    z_deb= z_deb+sigma_deb*K(v_deb);
    
    %% Support detection
    
    
    norm_z=compute_norm_z(z);
    support=norm_z>lambda*(1+1e-10);
    
    
    
    % prevent from divisions by 0
    norm_z(norm_z<eps)=eps;
    
    % get normal direction
    n_z=z./norm_z;
    
    if strcmp(block_penalty,'Bregman')
        n_z=z/lambda;
        n_z(support)=z(support)./norm_z(support);
    end
    
    if full_support
        support=norm_z>=0;
    end
    
    switch(block_penalty)
        case 'HO'
            proj_z_deb=proj_z(z_deb,n_z);
            z_deb(support)=z_deb(support)-proj_z_deb(support);
        case 'HD'
            proj_z_deb=proj_z(z_deb,n_z);
            index= support & repmat(sum(n_z.*z_deb,2), [1 nb_blocks])>=0;
            z_deb(index)=z_deb(index)-proj_z_deb(index);
        case 'QO'
            proj_z_deb=proj_z(z_deb,n_z);
            estim_norm=repmat(sqrt(sum((K(v)).^2,2)), [1 nb_blocks]);
            z_deb(support)=lambda*(z_deb(support)-proj_z_deb(support))./(lambda+sigma_deb.*estim_norm(support));
        otherwise % 'SD' and Bregman
            z_n_z=z_deb+lambda*n_z;
            norm_z_n_z=max(lambda,repmat(sqrt(sum((z_n_z).^2,2)), [1 nb_blocks]));
            z_deb(support)=lambda*(z_n_z(support)./norm_z_n_z(support)-n_z(support));
    end
    
    %% Proximal operator for TV^* on standard dual variable
    z=prox_z(z,norm_z,lambda,sigma);
    
    %% Primal step
    
    id_tauPhi_m1=1./(1+tau);
    if phi_not_id
        id_tauPhi_m1=inv(Id+tau*phitphi);
    end
    
    x=mul_phitphi(id_tauPhi_m1,x+tau*(phi_t_obs+KT(z)));
    
    id_tauPhi_m1=1./(1+tau_deb);
    if phi_not_id
        id_tauPhi_m1=inv(Id+tau_deb*phitphi);
    end
    x_deb=mul_phitphi(id_tauPhi_m1,x_deb+tau_deb*(phi_t_obs+KT(z_deb)));
    
    
    % acceleration
    
    theta=1./(sqrt(1+tau));
    tau=theta*tau;
    sigma=sigma/theta;
    
    theta_deb=1./(sqrt(1+tau));
    tau_deb=theta_deb*tau_deb;
    sigma_deb=sigma_deb/theta_deb;
    
    
    
    
    % upadate of auxiliary primal variables
    v=x+theta*(x-xold);
    v_deb=x_deb+theta_deb*(x_deb-xold_deb);
    
    
    
    % Display
    if mod(t-1,100)==0
        display_images(x_obs,x,x_deb,block_penalty)
    end
    
    
end