close all
clear all

%N. Papadakis, 2019

% Restore image y with Isotropic TV 
%\hat x=argmin_x ||\phi x-y||^2+\lambda ||\nabla x||_{1,2}

%and estimate the re-fitting of this estimation (loss of contrast correction and recovering of missing details) with a block penalty BP 

%||\phi x_deb -y||^2+\lambda BP(x_deb, hat x)


% If you you this code, please cite:
% C.A. Deledalle, N. Papadakis, J. Salmon and S. Vaiter. 
% Refitting solutions promoted by l_12 sparse analysis regularization with block penalties. 
% In Proc. of International Conference on Scale Space and Variational
% Methods (SSVM), pp 131-143, 2019


%% Parameters

lambda= .3;%   % regularization parameter
nb_iteration=1000; %nb of iteration for the iterative algorithm

% Choice of Block penalty BP for refitting
list_block_penalty={'SD','HO','HD','QO','Bregman'};
block_penalty=list_block_penalty{1};

% References:
% SD : %C.A. Deledalle, N. Papadakis, J. Salmon and S. Vaiter. Refitting solutions promoted by l_12 sparse analysis regularization with block penalties. In Proc of International Conference on Scale Space and Variational Methods (SSVM), pp 131-143, 2019
% HO and HD: E.-M. Brinkmann, M. Burger, J. Rasch, and C. Sutour. Bias-reduction in variational regularization. J. of Math. Imaging and Vision, 2017.
% QO: C.-A. Deledalle, N. Papadakis, J. Salmon, and S. Vaiter. CLEAR: Covariant least square  re-fitting with applications to image restoration. SIAM J. on Imaging Sciences, 10(1):243?284, 2017
% Bregman: S. Osher, M. Burger, D. Goldfarb, J. Xu, and W. Yin. An iterative regularization method for total variation-based image restoration. Multiscale Model. & Simul., 4(2):460?489, 2005.





%% Load Data and define operator

name_image='cameraman.tif'; 
%% test on color image
%name_image='peppers.png';

im=double(imread(name_image))/255.;
noise_level=20./255.;  %noise for observations

x_obs=im+randn(size(im))*noise_level;

phi=1; %no operator phi
% in this implementation, phi should be a square matrix of size:(Nx*Ny,Nx*Ny)

%% exemple with inpaiting problem, when only a subset of pixels are observed
% [Nx, Ny, Ncolor]=size(x_obs);
% phi=speye(Nx*Ny,Nx*Ny);  
% mask=double(rand(Nx,Ny)>0.4);
% phi=spdiags(mask(:),0,Nx*Ny,Nx*Ny);
% x_obs=reshape(phi*reshape(x_obs, [Nx*Ny Ncolor]), [Nx Ny Ncolor]);

[x, x_deb]=perform_refitting(x_obs,phi,lambda,block_penalty,nb_iteration);

display_images(x_obs,x,x_deb,block_penalty);